package EZeit;
#=========================================================================
#
#    EZeit.pm
#
#    Module collecting all functions necessary to operate on the EZeit
#    database including interaction with ZDB and reading/writing data
#    to the MySQL.
#
#    $Id: $
#    Last change: <Tue, 2009/01/06 18:20:48 arwagner ingata>
#    Author     : Alexander Wagner
#    Language   : Perl
#
#-------------------------------------------------------------------------
#
#    Copyright (C) 2008 by Alexander Wagner
#
#    This is free software; you can redistribute it and/or modify it
#    under the terms of the GNU Genereal Public License as published
#    by the Free Software Foundation; either version 2, or (at your
#    option) any later version.
#
#    This program is distributed in the hope that it will be usefull,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#    General Public License for more details.
#
#    You should have recieved a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#=========================================================================

use Exporter;
use vars qw($VERSION @ISA @EXPORT);
$VERSION = "0.01";

use XML::XPath;
use DBI;
use PICA::PSI::HTML;

@ISA    = ('Exporter');
@EXPORT = qw(&trim &Notify &Error &Marker &AddSQLString);

#======================================================================
$DEBUG           = 0;
$enableZDB       = 1;        # set to 0 to disable ZDB queries

# ZDB OPAC connection
$zdbhost         = "http://dispatch.opac.d-nb.de/DB=1.1/";

# connection to the MySQL
$user            = "";
$pass            = "";
$dbname          = "";
$dbhost          = "";

$year            = 2000;     # Year of the statistics to read

# From where to read data and where to write the query results
$inputdir        = "";
$outputdir       = "";

@years           = ();       # Years to process

@j1files         = ();       # J1 report files to process
@d1files         = ();       # D1 report files to process
@d3files         = ();       # D3 report files to process

@ezbfiles        = ();       # Reports from EZB

# Define how extensive the notification markers should be
$markerstring    = ".cUZ+";

$NLOffset        = 99990000;

#======================================================================

=head1 NAME

ezeit - routines for ezeit application

#----------------------------------------------------------------------
=head2 trim

Remove blacks from a string

Status: checked

=cut
#----------------------------------------------------------------------
sub trim {
   my @text = @_;
   for (@text) {
      s/^\s+//g;
      s/\s+$//;
   }
   return wantarray ? @text : $text[0];
}

#----------------------------------------------------------------------
=head2 Marker

Print a marker without CR

$string : the marker to be printed if set in $markerstring

Status: checked

=cut
#----------------------------------------------------------------------
sub Marker($) 
{
   my ($string) = @_;
   print "$string" if ($markerstring =~ m/$string/);
   
}

#----------------------------------------------------------------------
=head2 Notify

Print msg to stdout if debugging is enabled

$msg: The notification message to print

Status: checked

=cut
#----------------------------------------------------------------------
sub Notify($) 
{
   my ($string) = @_;
   if ($DEBUG > 0) {
      print "$string\n";
   }
}

#----------------------------------------------------------------------
=head2 Error

Print msg to stdout prepended by [ERROR], and die if debug

$msg: Error message to print

Status: checked

=cut
#----------------------------------------------------------------------
sub Error($) 
{
   my ($string) = @_;
   if ($DEBUG > 9) {
      die "$string\n";
   }
   else {
      print "[ERROR] $string\n";
   }
}

#----------------------------------------------------------------------
=head2 AddSQLString

Quote the string and add the separator

$value    : The value to be quoted for SQL
$separator: The separator, e.g. ", " or ");"

Status: checked

=cut
#----------------------------------------------------------------------
sub AddSQLString($$) {
   my ($value, $separator) = @_;

   if (defined($value)) {
      return "\"$value\"$separator";
   }
   else {
      return "\"\"$separator";
   }
}

#----------------------------------------------------------------------
=head2 ReadConfig

Read in the config file (ezeit.xml) using XPath for parsing

Status: checked

=cut
#----------------------------------------------------------------------
sub ReadConfig() {
   my $xmlfile = "ezeit.xml";
   my $xp = XML::XPath->new(filename=>$xmlfile);

   my $nodeset = $xp->find('/ezeit');

   foreach my $node ($nodeset->get_nodelist) {
      $user         = "" . $node->findvalue('db/user');
      $pass         = "" . $node->findvalue('db/pass');
      $dbname       = "" . $node->findvalue('db/dbname');
      $dbhost       = "" . $node->findvalue('db/dbhost');
      $zdbhost      = "" . $node->findvalue('zdb/host');
      $enableZDB    = "" . $node->findvalue('zdb/enable');
      $inputdir     = "" . $node->findvalue('input/inputdir');
      $outputdir    = "" . $node->findvalue('output/outputdir');
      $markerstring = "" . $node->findvalue('input/markerstring');
   }

   for (my $i=1; $i <= $xp->find('count(/ezeit/input/time/year)'); $i++) {
      $years[$i-1]   = $xp->find("/ezeit/input/time/year[$i]");
   }
   for (my $i=1; $i <= $xp->find('count(/ezeit/input/j1/file)'); $i++) {
      $j1files[$i-1] = $xp->find("/ezeit/input/j1/file[$i]");
   }
   for (my $i=1; $i <= $xp->find('count(/ezeit/input/d1/file)'); $i++) {
      $d1files[$i-1] = $xp->find("/ezeit/input/d1/file[$i]");
   }
   for (my $i=1; $i <= $xp->find('count(/ezeit/input/d3/file)'); $i++) {
      $d3files[$i-1] = $xp->find("/ezeit/input/d3/file[$i]");
   }
   for (my $i=1; $i <= $xp->find('count(/ezeit/input/ezb/file)'); $i++) {
      $ezbfiles[$i-1] = $xp->find("/ezeit/input/ezb/file[$i]");
   }
}

#----------------------------------------------------------------------
=head2 ConnectDB

Connect to MySQL

Return: $dbh, the database handle for the connection

Status: checked

=cut
#----------------------------------------------------------------------
sub ConnectDB() {
   my $dsn = "DBI:mysql:$dbname:$dbhost:3306";

   my $dbh    = DBI->connect($dsn, $user, $pass 
                           , {RaiseError => 0, PrintError => 0}
               )
      or print "Connection to DB failed: $DBI::errstr\n";

   return($dbh);
}

#----------------------------------------------------------------------
=head2 ExecuteSQL

Execute an SQL statement, that does not return a resultset.

$dbh: Database handle to use
$sql: The SQL-command to execute

Status: checked

=cut
#----------------------------------------------------------------------
sub ExecuteSQL($$) {
   my ($dbh, $sql) = @_;

   my $sth = $dbh->prepare($sql);
   $sth->execute;

   $sth->finish;
}

#----------------------------------------------------------------------
=head2 IsNumeric

Check if an expression represents a valid number

$str: The string that should be checked

Status: checked

=cut
#----------------------------------------------------------------------
sub IsNumeric {
   my $val = $_[0];
   my $DIGITS = qr{ \d+ (?: [.] \d*)? | [.] \d+ }xms;
   my $SIGN   = qr{ [+-] }xms;
   my $NUMBER = qr{ ($SIGN?) ($DIGITS) }xms;
   if ( $val !~ /^${NUMBER}$/ ) {
      return 0;   #Non-numeric
   } else {
      return 1;   #Numeric
   }
}

#======================================================================
#
# Updating local DB
#
#----------------------------------------------------------------------

#----------------------------------------------------------------------
=head2 AddNLSubscription

Treat journal subscriptions by National Licensing.

$dbh     : database handle
$record  : Pica record from PicaPM
$ezbid   : EZB-ID of the journal
$NLStart : Start of the National License (year)
$NLEnd   : End of the National License (e.g. current year)

Status: checked

=cut
#----------------------------------------------------------------------
sub AddNLSubscription($$$$$)
{
   my ($dbh, $zdbid, $ezbid, $NLStart, $NLEnd) = @_;
 
   my $sql = "";
 
   Marker("N");

   $sql  = "INSERT INTO NL (";
   $sql .= "zdbid, ezbid, startyear, endyear) VALUES (";
   $sql .= AddSQLString($zdbid, ", ");
   $sql .= AddSQLString($ezbid, ", ");
   $sql .= AddSQLString($NLStart, ", ");
   $sql .= AddSQLString($NLEnd, ");");

   $dbh->do($sql);
}

sub StoreISSNList($$$$)
{
   my ($dbh, $ezbid, $printissn, $onlineissn) = @_;

   # store only those ISSNs that where delivered by the publisher
   if (($printissn ne "") and ($ezbid > 0)) {
      $sql = "INSERT INTO issnlist (ezbid, issn) VALUES (";
      $sql = $sql . AddSQLString($ezbid, ", ");
      $sql = $sql . AddSQLString($printissn, ");");
      $dbh->do($sql);
   }

   if (($onlineissn ne "") and ($ezbid > 0)) {
      $sql = "INSERT INTO issnlist (ezbid, issn) VALUES (";
      $sql = $sql . AddSQLString($ezbid, ", ");
      $sql = $sql . AddSQLString($onlineissn, ");");
      $dbh->do($sql);
   }
}

#----------------------------------------------------------------------
=head2 UpdateJournalTable

Store data from the provider to the database

$dbh        : database handle
$table      : tablename for update
$journal    : journal name
$publisher  : publisher of the journal
$onlineissn : ISSN of the online issue
$printissn  : ISSN of the corresponding print issue
$platform   : name of the platform
$ezbid      : EZB-ID

Status: checked

=cut
#----------------------------------------------------------------------
sub UpdateJournalTable($$$$$$$$)
{
   my ($dbh, $table, $journal, $publisher, $onlineissn, 
       $printissn, $platform, $ezbid) = @_;

   $sql = "SELECT COUNT(ezbid) FROM $table j ";
   $sql = $sql . "WHERE (j.ezbid    = \"$ezbid\") ";
   $sql = $sql . "  AND (j.platform = \"$platform\")";
   $sth = $dbh->prepare($sql);
   $sth->execute;
   if (@row = $sth->fetchrow_array) {
      $count = $row[0];
   }

   if (($count == 0) or ($table =~ m/^unresolved/)) {
      Marker("j");

      # ISSNs are stored her only to preserve the "original record"
      # Do NOT use them for queries!
      $sql = "INSERT INTO $table (";
      $sql = $sql . "name, publisher, issn_online, ";
      $sql = $sql . "issn_print, platform, ezbid";
      $sql = $sql . ") VALUES (";
      $sql = $sql . AddSQLString($journal, ", ");
      $sql = $sql . AddSQLString($publisher, ", ");
      $sql = $sql . AddSQLString($onlineissn, ", ");
      $sql = $sql . AddSQLString($printissn, ", ");
      $sql = $sql . AddSQLString($platform, ", ");
      $sql = $sql . AddSQLString($ezbid, "");
      $sql = $sql . ");";
      $dbh->do($sql);
   }
   StoreISSNList($dbh, $ezbid, $printissn, $onlineissn);
   $sth->finish;
}

#----------------------------------------------------------------------
=head2 UpdateYearUsage

Update the yearly usage for $ezbid/$platform
Note: $zdbid can also be the non-zdbid

$dbh      : database handle
$ezbid    : EZB-ID
$platform : platform
$year     : year of the data
$YTDHTML  : HTML accesses for $year
$YTDPDF   : PDF accesses for $year
$YTD      : total accesses for $year
$donenum  : already done (0/1)? then increment current value 

=cut
#----------------------------------------------------------------------

sub UpdateYearUsage($$$$$$$$)
{
   my ($dbh, $ezbid, $platform, $year, $YTDHTML, $YTDPDF, $YTD, $donenum) = @_;
   my $cound = 0;
   my $sth;

   $sql = "SELECT COUNT(ezbid) FROM Jonlineyearly ";
   $sql = $sql . "WHERE (ezbid=\"$ezbid\") AND (platform=\"$platform\")";
   $sql = $sql . "AND (year = \"$year\") ";
   $sth = $dbh->prepare($sql);
   $sth->execute;
   if (@row = $sth->fetchrow_array) {
      $count = $row[0];
   }

   if ($count == 0) {
      Marker("Y");
      $sql = "INSERT INTO Jonlineyearly ";
      $sql = $sql . "(ezbid, platform, year, html, pdf, total) ";
      $sql = $sql . "VALUES (";
      $sql = $sql . AddSQLString($ezbid, ", ");
      $sql = $sql . AddSQLString($platform, ", ");
      $sql = $sql . AddSQLString($year, ", ");
      $sql = $sql . AddSQLString($YTDHTML, ", ");
      $sql = $sql . AddSQLString($YTDPDF, ", ");
      $sql = $sql . AddSQLString($YTD   , "");
      $sql = $sql . ");";
      $dbh->do($sql);
   }
   else {
      Marker("y");
      if ($donenum > 1) {
         # add to current values
         $sql = "UPDATE Jonlineyearly SET ";
         $sql = $sql . "html= html + $YTDHTML, ";
         $sql = $sql . "pdf=pdf + $YTDPDF, ";
         $sql = $sql . "total=total + $YTD";
         $sql = $sql . "WHERE ";
         $sql = $sql . "(platform=" . AddSQLString($platform, ") AND ");
         $sql = $sql . "(ezbid="    . AddSQLString($ezbid,    ") AND ");
         $sql = $sql . "(year="     . AddSQLString($year,     ");");
         $dbh->do($sql);
      }
      else {
         $sql = "UPDATE Jonlineyearly SET ";
         $sql = $sql . "html="  . AddSQLString($YTDHTML, ", ");
         $sql = $sql . "pdf="   . AddSQLString($YTDPDF, ", ");
         $sql = $sql . "total=" . AddSQLString($YTD   , " ");
         $sql = $sql . "WHERE ";
         $sql = $sql . "(platform=" . AddSQLString($platform, ") AND ");
         $sql = $sql . "(ezbid="    . AddSQLString($ezbid,    ") AND ");
         $sql = $sql . "(year="     . AddSQLString($year,     ");");
         $dbh->do($sql);
      }
   }
   $sth->finish;
}

#----------------------------------------------------------------------
=head2 UpdateMonthUsage

Update the monthly usage for $ezbid, $platform

$dbh      : database handle
$ezbid    : EZB-ID to update
$platform : platform
$year     : year of the data to add/update
$donenum  : already done (0/1)? then increment current value 
@usage    : array of monthly usages starting in january

=cut
#----------------------------------------------------------------------
sub UpdateMonthUsage($$$$$@)
{
   my($dbh, $ezbid, $platform, $year, $donenum, @usage) = @_;
   my $sth;

   Marker("m");

   for (my $i=0; $i < $#usage; $i++) {
      my $count = 0;
      $month = $i+1;
      $timestring = $year . "-" . $month . "-01";

      $sql = "SELECT COUNT(ezbid) FROM Jonlinemonthly ";
      $sql = $sql . "WHERE ezbid = \"$ezbid\" AND ";
      $sql = $sql . "timeframe = \"$timestring\";";
      $sth = $dbh->prepare($sql);
      $sth->execute;
      if (@row = $sth->fetchrow_array) {
         $count = $row[0];
      }

      if ($count == 0) {
         $sql = "INSERT INTO Jonlinemonthly(";
         $sql = $sql . "ezbid, platform, timeframe, counter";
         $sql = $sql . ") ";
         $sql = $sql . "VALUES (";
         $sql = $sql . AddSQLString($ezbid, ", ");
         $sql = $sql . AddSQLString($platform, ", ");
         $sql = $sql . AddSQLString($timestring, ", ");
         $sql = $sql . AddSQLString($usage[$i], "");
         $sql = $sql . ");";
         $dbh->do($sql);
      }
      else {
         if ($donenum > 1) {
            # add to current value
            $sql = "UPDATE Jonlinemonthly SET ";
            $sql = $sql . "counter=counter + $usage[$i]";
            $sql = $sql . "WHERE ";
            $sql = $sql . "(platform=" . AddSQLString($platform,   ") AND ");
            $sql = $sql . "(ezbid="    . AddSQLString($ezbid,      ") AND ");
            $sql = $sql . "(timeframe=". AddSQLString($timestring, ");");
            $dbh->do($sql);
         }
         else {
            $sql = "UPDATE Jonlinemonthly SET ";
            $sql = $sql . "counter="   . AddSQLString($usage[$i], " ");
            $sql = $sql . "WHERE ";
            $sql = $sql . "(platform=" . AddSQLString($platform,   ") AND ");
            $sql = $sql . "(ezbid="    . AddSQLString($ezbid,      ") AND ");
            $sql = $sql . "(timeframe=". AddSQLString($timestring, ");");
            $dbh->do($sql);
         }
      }
   }
   $sth->finish;
}

#
#======================================================================
#
# Stuff related to ZDB usage
#
#----------------------------------------------------------------------
=head2 ConnectZDB

Connect to ZDB and return a PICA::PSI::HTML structure with a proper
handle for subsequent usage.

Status: checked

=cut
#----------------------------------------------------------------------
sub ConnectZDB() {
   return(PICA::PSI::HTML->new($zdbhost));
}

#----------------------------------------------------------------------
=head2 ISSN2ZDBPPN

Retrieve PPNs for a given ISSN from the ZDB OPAC

$zdbopac: URL of the ZDB OPAC (Web)
$issn   : ISSN to look up

return: @PPN, array of PPNs matching the ISSN

Status: checked

=cut
#----------------------------------------------------------------------
sub ISSN2ZDBPPN($$)
{
   my ($zdbopac, $issn) = @_;

   Marker("z");
   @PPN = $zdbopac->fetchPPNListByXML("SRCHM&ACT=SRCH&IKT=8&TRM=$issn");
   return(@PPN);
}

#----------------------------------------------------------------------
=head2 PPN2ZDBRecord

ZDB-Record for a given PPN as PICA-structure.

$zdbopac: URL of the ZDB OPAC (Web)
$ppn    : PPN to fetch bibliographic data

return: $record, PicaPM record

Status: checked

=cut
#----------------------------------------------------------------------
sub PPN2ZDBRecord($$)
{
   my ($zdbopac, $ppn)      = @_;

   my $record = $zdbopac->fetchPPN($ppn);
   return($record)
}

#----------------------------------------------------------------------
=head2 ZDBSubjects

Store Subjects and the EZB<->ZDB assocs

Status: checked

=cut
#----------------------------------------------------------------------
sub ZDBSubjects($$$)
{
   my ($dbh, $record,$ezbid) = @_;

   if ($ezbid > 0) {
      my @zdbcodes = $record->subfield('045U$a');
      foreach $subj (@zdbcodes) {
         $sql = "INSERT INTO Jsubjects (";
         $sql = $sql . "zdbid, ezbid, subjectgroup ";
         $sql = $sql . ") VALUES (";
         $sql = $sql . AddSQLString($record->subfield('006Z$0'), ", ");
         $sql = $sql . AddSQLString($ezbid, ", ");
         $sql = $sql . AddSQLString($subj, "");
         $sql = $sql . ");";

         $dbh->do($sql);
      }
   }
}

#----------------------------------------------------------------------
=head2 ExtractEZBIDNL

Extracts the EZBID and the NL-Status from a given PICA-record structure

$record: a PicaPM-record to extract data from

return: 
   $ezbid  : EZB-ID of the record
   $NL     : 1 if nationally licensed
   $NLStart: start issue (year) of national license
   $NLEnd  : end issue (year) of national license
Status: 

=cut
#----------------------------------------------------------------------
sub ExtractEZBIDNL($)
{
   my ($record) = @_;
   my $ezbid    = 0;
   my $NL       = 0;
   my $NLStart = 0;
   my $NLEnd   = 0;
   my $NLTime  = "";

   if (defined($record->field('009Q'))) {
      my @links = $record->field('009Q');
      foreach $link (@links) {
         my $url = $link->subfield("u");
         my $type = $link->subfield("x");
         # Link to Elektronische Zeitschriften Regensburg
         if ($type eq "F") {
            $ezbid = $url;
            $ezbid =~ s#http://www.bibliothek.uni-regensburg.de/ezeit/\?##;
         }
         # First possibility to detect a national license
         if (defined($link->subfield("z"))) {
            if ($link->subfield("z") eq "NL") {
               # this procedure is not really reliable but there is no
               # better data @ZDB and all entries are treated as text
               # there, so not necessarily years are numbers...
               $NL = 1;

               $NLTime = $link->subfield("x");
               $NLTime =~ s/^H; //;
               $NLTime =~ s#\[\?\]##;

               $NLStart = $NLTime;
               $NLEnd   = $NLTime;

               $NLStart =~ s#^[0-9]*\.##;
               ## to handle 1937/38 and similar.
               $NLStart =~ s#/.*##;
               $NLStart =~ s# -.*##;

               $NLEnd   =~ s#[0-9.]* -##;
               $NLEnd   =~ s#^[ 0-9]*\.##;
               $NLEnd   =~ s#/.*##;

               $NLStart = trim($NLStart);
               $NLEnd   = trim($NLEnd);

               ($second, $minute, $hour, $dayOfMonth, $month,
                  $yearOffset, $dayOfWeek, $dayOfYear, $daylightSavings)
               = localtime();
               $year = 1900 + $yearOffset;

               if ($NLEnd eq "") {
                  $NLEnd = $year;
               } 

               if ($NLEnd =~ m/-1 Jahr/) {
                  $NLEnd = $year-1;
               }

               if ($NLStart =~ /[^0-9]/) {
                  $NLStart = 0;
               }
               if ($NLEnd =~ /[^0-9]/) {
                  $NLEnd = 0;
               }
            }
         }
      }
      # Second possibility to detect a national license
      # But: Subfield $a is "repeatable"??? (not documented)
      if (defined($record->subfield('017A$a'))) {
         $NL = 1 if ($record->subfield('017A$a') eq "nl");
      }

      return($ezbid, $NL, $NLStart, $NLEnd)
   }
   else {
      return(0, 0, 0, 0)
   }

}

#----------------------------------------------------------------------
=head2 ZDBRecord2DBI

Store the ZDB-Data on the local database

$dbh     : database handle
$record  : PicaPM record to add
$ezbid   : EZB-ID

return: $ezbid, the EZB-ID

Status: checked

=cut
#----------------------------------------------------------------------
sub ZDBRecord2DBI ($$$)
{
   my ($dbh, $record,$ezbid) = @_;
   my $NL              = 0;
   my $dummy           = 0;

   if (($record->subfield('002@$0') =~ m/^A/) or
       ($record->subfield('002@$0') =~ m/^O/) or
       ($record->subfield('002@$0') =~ m/^S/)) {

      ($dummy, $NL, $NLStart, $NLEnd) = ExtractEZBIDNL($record);


      $sql = "INSERT INTO ZDB (";
      $sql = $sql . "zdbid, ppn, matcode, ";
      $sql = $sql . "title, subseries, publisher, ";
      $sql = $sql . "shorttitle, lang, yearstart, yearend, ";
      $sql = $sql . "coden, dfglicense";
      $sql = $sql . ") VALUES (";
      $sql = $sql . AddSQLString($record->subfield('006Z$0'), ", ");
      $sql = $sql . AddSQLString($record->subfield('003@$0'), ", ");
      $sql = $sql . AddSQLString($record->subfield('002@$0'), ", ");
      $sql = $sql . AddSQLString($record->subfield('021A$a'), ", ");
      $sql = $sql . AddSQLString($record->subfield('021C$r'), ", ");
      $sql = $sql . AddSQLString($record->subfield('021A$h'), ", ");
      $sql = $sql . AddSQLString($record->subfield('026C$a'), ", ");
      $sql = $sql . AddSQLString($record->subfield('010@$a'), ", ");
      if (defined($record->subfield('011@$a'))) {
         $sql = $sql . AddSQLString($record->subfield('011@$a'), ", ");
      }
      else {
         $sql = $sql . AddSQLString("", ", ");
      }
      if (defined($record->subfield('011@$b'))) {
         $sql = $sql . AddSQLString($record->subfield('011@$b'), ", ");
      }
      else {
         $sql = $sql . AddSQLString("", ", ");
      }
      $sql = $sql . AddSQLString($record->subfield('007C$0'), ", ");
      $sql = $sql . AddSQLString($NL, "");
      $sql = $sql . ");";

      $dbh->do($sql);

      ZDBSubjects($dbh, $record, $ezbid);
      if (($NL > 0) and (IsNumeric($NLStart)) and (IsNumeric($NLEnd))) {
         AddNLSubscription($dbh, $zdbid, $ezbid, $NLStart, $NLEnd);
      }
   }
   return($ezbid);
}

#----------------------------------------------------------------------
=head2 UpdateFromZDB:

Retrieve data for an ISSN given from ZDB and insert into the ZDB table
on mysql. This code does not account for predecessors and successors!
Only parallel editions are collected.


$dbh   : database handle
$zdb   : url of the ZDB OPAC (web)
@issn  : array of exaclty 2 issns, one for electronic, one for print

Status: checked

=cut
#----------------------------------------------------------------------

sub UpdateFromZDB($$$$)
{
   my ($dbh, $zdb, @issns)  = @_;
   my $returnID = "";
   my $ezbid    = "0";
   my $dummy    = "";
   my $record;
   my %records  = ();
   my %seen     = ();
   my $rec;

   Marker("Z");
   foreach $issn (@issns) {
      if ($issn ne "") {
         @PPNList = ISSN2ZDBPPN($zdb, $issn);
         foreach $PPN (@PPNList){
            $records{$PPN} = 1;  
         }

         foreach $PPN (@PPNList) {
            Marker(":");
            $seen{$PPN} = 1;
            $record = PPN2ZDBRecord($zdb, $PPN);
            $records{$PPN} = $record;

            if (defined($record)) {

               if ($ezbid < 1) {
                  ($ezbid, $dummy) = ExtractEZBIDNL($record);
               }

               # parallel editions
               if (defined($record->field('039D'))) {
                  @parallel = $record->field('039D');
                  foreach $par(@parallel) {
                     if (defined($par->subfield('$9'))) {
                        $ppn = $par->subfield('$9');
                        unless($seen{$ppn}) {
                           $seen{$ppn} = 1;
                           push(@PPNList, $ppn);
                        }
                     }
                  }
               }
               # predecessors and successors
               if (defined($record->field('039E'))) {
                  @predsucc = $record->field('039E');
                  foreach $ps (@predsucc) {
                     if (defined($ps->subfield('$9'))) {
                        $ppn = $ps->subfield('$9');
                        unless($seen{$ppn}) {
                           $seen{$ppn} = 1;
                           push(@PPNList, $ps->subfield('$9'));
                        }
                     }
                  }
               }
            }
         }

         foreach $PPN (@PPNList) {
            ZDBRecord2DBI($dbh, $records{$PPN}, $ezbid);
         }
      }
   }
   return($ezbid)
}


#----------------------------------------------------------------------
=head2 GetLocalEZBID

Given the database handle and two ISSNs (online and print) the DB is
searched for the associated EZBID to avoid queries to ZDB.  Either
ISSN might be empty. It is assumed that the table "issnlist" is filled
with proper associations. If the query results in an EZBID it is
returned, an empty string otherwise.

$dbh   : database handle
@issn  : array of exaclty 2 issns, one for electronic, one for print

return: $ezbid, the EZB-ID retrieved from the local DB, ie. without
        query to ZDB OPAC

Status: checked

=cut
#----------------------------------------------------------------------
sub GetLocalEZBID($$$)
{
   my ($dbh, @issns) = @_;
   my $sth;
   my $i = 0;
   my $ezbid = 0;

   $sql = "SELECT ezbid FROM issnlist i WHERE ";
   $sql = $sql . "i.issn= \"" . $issns[0] . "\" ";
   $sql = $sql . "OR  ";
   $sql = $sql . "i.issn= \"" . $issns[1] . "\" ";
   $sql = $sql . "GROUP BY i.ezbid;";

   $sth = $dbh->prepare($sql);
   $sth->execute;
   if (@row = $sth->fetchrow_array) {
      if ($#row == 0) {
         $ezbid = $row[0];
      }
   }
   $sth->finish;

   return($ezbid);
}

#----------------------------------------------------------------------
=head2 GetEZBID

Given a DB handle and two ISSNs reslove to an EZBID. To this end first
check the local DB if a proper association is known. If that fails ask
ZDB for help if $enableZDB is set.

$dbh   : database handle
$zdb   : URL of the ZDB OPAC (web)
@issn  : array of exaclty 2 issns, one for electronic, one for print

Status: checked

=cut
#----------------------------------------------------------------------
sub GetEZBID($$$$)
{
   my ($dbh, $zdb, @issns) = @_;

   my $ezbid = GetLocalEZBID($dbh, $issns[0], $issns[1]);

   if (($ezbid < 1) and ($enableZDB)) {
      $ezbid = UpdateFromZDB($dbh, $zdb, $issns[0], $issns[1]);
      StoreISSNList($dbh, $ezbid, $issns[0], $issns[1]);
   }
   return($ezbid);
}

#----------------------------------------------------------------------
=head2 GetDBsubsid

Query the DB for a subsid of a database. Query has to be performed via
database name and platform as no unique identifier exists.

$dbh      : database handle
$name     : name of the database to retrieve subsid (e.g. Inspec)
$platform : host platform of the DB $name (e.g. Ovid)

Status: unfinished

=cut
#----------------------------------------------------------------------

sub GetDBsubsid($$$)
{
   my ($dbh, $name, $platform) = @_;
   
   my $id = 0;

   $sql  = "SELECT DBN.subsid ";
   $sql .= "FROM DBName DBN, DB WHERE ";
   $sql .= "((DBN.platform rlike \"$platform\") AND (DB.name rlike \"$name\"));";

   $sth = $dbh->prepare($sql);
   $sth->execute;
   if (@row = $sth->fetchrow_array) {
      $id = $row[0];
   }
   $sth->finish;
   return($id);
}

#----------------------------------------------------------------------
=head2 DBUpdateYear

Updates the year counters for databases either in searches or sessions

$dbh    : Database handle
$DBName : Name prefix ("Searches" or "Sessions")
$subsid : subscription ID
$name   : Name of the database to add stats to
$year   : Year of the stats
$counter: Number to add

Status: checked

=cut
#----------------------------------------------------------------------
sub DBUpdateYear($$$$$$)
{
   my ($dbh, $DBName, $subsid, $name, $year, $counter) = @_;
   my $count = 0;
   my $sth;

   $sql = "SELECT COUNT(subsid) FROM $DBName"."Year";
   $sql = $sql . "WHERE (subsid=\"$subsid\") AND (name=\"$name\")";
   $sql = $sql . "AND (year = \"$year\") ";
   $sth = $dbh->prepare($sql);
   $sth->execute;
   if (@row = $sth->fetchrow_array) {
      $count = $row[0];
   }

   if ($count == 0) {
      Marker("Y");
      $sql  = "INSERT INTO $DBName"."Year";
      $sql .= "(subsid, name, year, counter) ";
      $sql .= "VALUES (";
      $sql .= AddSQLString($subsid, ", ");
      $sql .= AddSQLString($name, ", ");
      $sql .= AddSQLString($year, ", ");
      $sql .= AddSQLString($counter, "");
      $sql .= ");";
      $dbh->do($sql);
   }
   else {
      Marker("y");
      $sql  = "UPDATE $DBName"."Year SET ";
      $sql .= "counter="  . AddSQLString($YTDHTML, ", ");
      $sql .= "WHERE ";
      $sql .= "(subsid="   . AddSQLString($subsid,   ") AND ");
      $sql .= "(name="     . AddSQLString($name,     ");");
      $dbh->do($sql);
   }
   $sth->finish;
}

#----------------------------------------------------------------------
=head2 DBUpdateMonth

Updates the month counters for databases either in searches or sessions

$dbh    : Database handle
$DBName : Name prefix ("Searches" or "Sessions")
$subsid : subscription ID
$name   : Name of the database to add stats to
$year   : Year of the stats
@usage  : numbers to add (monthly array)

Status: checked

=cut
#----------------------------------------------------------------------
sub DBUpdateMonth($$$$$@)
{
   my ($dbh, $DBName, $subsid, $name, $year, @usage) = @_;
   my $sth;

   Marker("m");

   for (my $i=0; $i < $#usage; $i++) {
      my $count = 0;
      $month = $i+1;
      $timestring = $year . "-" . $month . "-01";

      $sql = "SELECT COUNT(subsid) FROM $DBName". "Month ";
      $sql .= "WHERE subsid = \"$subsid\" AND ";
      $sql .= "name = \"$name\" AND ";
      $sql .= "timeframe = \"$timestring\";";
      $sth = $dbh->prepare($sql);
      $sth->execute;
      if (@row = $sth->fetchrow_array) {
         $count = $row[0];
      }

      if ($count == 0) {
         $sql  = "INSERT INTO $DBName" . "Month(";
         $sql .= "subsid, name, timeframe, counter";
         $sql .= ") ";
         $sql .= "VALUES (";
         $sql .= AddSQLString($subsid,     ", ");
         $sql .= AddSQLString($name,       ", ");
         $sql .= AddSQLString($timestring, ", ");
         $sql .= AddSQLString($usage[$i],  "");
         $sql .= ");";
         $dbh->do($sql);
      }
      else {
         $sql  = "UPDATE $DBName" . "Month SET ";
         $sql .= "counter="   . AddSQLString($usage[$i], " ");
         $sql .= "WHERE ";
         $sql .= "(name="     . AddSQLString($name,       ") AND ");
         $sql .= "(subsid="   . AddSQLString($subsid,     ") AND ");
         $sql .= "(timeframe=". AddSQLString($timestring, ");");
         $dbh->do($sql);
      }
   }
   $sth->finish;
}

#----------------------------------------------------------------------
=head2 DBUpdateSearches

Updates the search counters for databases

$dbh    : Database handle
$subsid : subscription ID
$name   : Name of the database to add stats to
$year   : Year of the stats
$YTD    : Years total
@usage  : Monthly numbers

Status: unfinished

=cut
#----------------------------------------------------------------------
sub DBUpdateSearches($$$$$@)
{
   my ($dbh, $subsid, $name,  $year, $YTD, @usage) = @_;

   DBUpdateYear ($dbh, "DBSearches", $subsid, $name, $year, $YTD);
   DBUpdateMonth($dbh, "DBSearches", $subsid, $name, $year, @usage);
}

#----------------------------------------------------------------------
=head2 DBUpdateSessions

Updates the session counters for databases

$dbh    : Database handle
$subsid : subscription ID
$name   : Name of the database to add stats to
$year   : Year of the stats
$YTD    : Years total
@usage  : Monthly numbers

Status: unfinished

=cut
#----------------------------------------------------------------------
sub DBUpdateSessions($$$$$@)
{
   my ($dbh, $subsid, $name,  $year, $YTD, @usage) = @_;

   DBUpdateYear ($dbh, "DBSessions", $subsid, $name, $year, $YTD);
   DBUpdateMonth($dbh, "DBSessions", $subsid, $name, $year, @usage);
}


#----------------------------------------------------------------------
=head2 UpdateEZBTable

Add entries to the EZB table for usage statistics from the EZB itself.

Status: checked

=cut
#----------------------------------------------------------------------
sub UpdateEZBTable($$$$)
{
   my ($dbh, $internid, $ezbid, $access) = @_;

   $sql = "SELECT COUNT(ezbid) FROM EZB j ";
   $sql = $sql . "WHERE (j.ezbid    = \"$ezbid\");";
   $sth = $dbh->prepare($sql);
   $sth->execute;
   if (@row = $sth->fetchrow_array) {
      $count = $row[0];
   }

   if ($count == 0) {
      Marker("j");

      # ISSNs are stored her only to preserve the "original record"
      # Do NOT use them for queries!
      $sql = "INSERT INTO EZB (";
      $sql = $sql . "internid, ezbid, access";
      $sql = $sql . ") VALUES (";
      $sql = $sql . AddSQLString($internid, ", ");
      $sql = $sql . AddSQLString($ezbid, ", ");
      $sql = $sql . AddSQLString($access, ");");
      $dbh->do($sql);
   }

   $sth->finish;
}

#----------------------------------------------------------------------
=head2 UpdateEZBUsage

Update the yearly usage of EZB for internal EZB-ID (internid)

$dbh      : database handle
$ezbid    : EZB-ID
$year     : year of the data
$YTD      : total accesses for $year

=cut
#----------------------------------------------------------------------

sub UpdateEZBUsage($$$$)
{
   my ($dbh, $internid, $year, $YTD) = @_;
   my $cound = 0;
   my $sth;

   $sql = "SELECT COUNT(internid) FROM EZBYearly ";
   $sql = $sql . "WHERE (internid=\"$internid\") ";
   $sql = $sql . "AND (year = \"$year\") ";
   $sth = $dbh->prepare($sql);
   $sth->execute;
   if (@row = $sth->fetchrow_array) {
      $count = $row[0];
   }

   if ($count == 0) {
      Marker("Y");
      $sql = "INSERT INTO EZBYearly";
      $sql = $sql . "(internid, year, total) ";
      $sql = $sql . "VALUES (";
      $sql = $sql . AddSQLString($internid, ", ");
      $sql = $sql . AddSQLString($year, ", ");
      $sql = $sql . AddSQLString($YTD   , "");
      $sql = $sql . ");";
      $dbh->do($sql);
   }
   else {
      Marker("y");
      $sql = "UPDATE EZBYearly SET ";
      $sql = $sql . "total=" . AddSQLString($YTD   , " ");
      $sql = $sql . "WHERE ";
      $sql = $sql . "(internid=" . AddSQLString($internid,    ") AND ");
      $sql = $sql . "(year="     . AddSQLString($year,     ");");
      $dbh->do($sql);
   }
   $sth->finish;
}


#----------------------------------------------------------------------
=head2

Status: unfinished

=cut
#----------------------------------------------------------------------


1;

__END__
